#include "QSudoku.h"

int main(int argc, char *argv[])
{
   int t = time(NULL);
   srand(t);
   
   QApplication app(argc, argv);
   QSudokuWindow main_window;
   return app.exec();
};

QSudokuWindow::QSudokuWindow(QWidget * parent, Qt::WFlags flags): QMainWindow(parent, flags)
{
   setWindowTitle(tr("QSudoku[*]"));
   setWindowIcon(QIcon(":/images/sudoku.png"));

   sudoku_widget = new QSudokuWidget(this);
   sudoku_engine = new Sudoku();
   description = new QLabel(tr(""),this);
   description->setScaledContents(true);
   
   description->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
   description->setAlignment(Qt::AlignLeft | Qt::AlignTop );   
   
  
   QWidget * base = new QWidget(this);
   QVBoxLayout * vl = new QVBoxLayout;
   vl->setMargin(0);
   vl->setSpacing(0);   
   vl->addWidget(sudoku_widget);
   vl->addWidget(description);   
   base->setLayout(vl);
   
   setCentralWidget(base);
  
   createActions();
   createMenus();
   createToolBars();
   createStatusBar();
   
   readSettings();
   
   connect(sudoku_widget, SIGNAL(contentsChanged()), this, SLOT(documentWasModified()));

   ChangeSolvedMode(false);
   show();

};

void QSudokuWindow::ChangeSolvedMode(bool is_solved)
{
   if (is_solved)
   {
      sudoku_widget->SetEditMode(false);
      solveAct->setEnabled(false);
      editAct->setEnabled(true);      
      saveWithSolutionAct->setEnabled(true);      
      showHistoryAct->setChecked(false);          
      if ( solution_count > 0 )
      {
         showHistoryAct->setEnabled(true);                
      }
   }
   else
   {
     int size = sudoku_widget->getSmallSquareSize()*sudoku_widget->getSmallSquareSize();
     for (int i = 0; i< size; i++)for (int j = 0; j< size; j++)
      {
         sudoku_widget->UpdateValue(i,  j, sudoku_widget->GetValue( i, j ), QSudokuWidget::empty);         
      }       
      
      sudoku_widget->SetEditMode(true);
      description->setText(tr("<B>Edit mode</B>"));   
      solveAct->setEnabled(true);
      editAct->setEnabled(false);  
      show1Act->setEnabled(false);        
      show2Act->setEnabled(false);              
      saveWithSolutionAct->setEnabled(false);
      excess_items.clear();
      solution_history.clear();
      solution_count = 0;
      showHistoryAct->setChecked(false);          
      showHistoryAct->setEnabled(false);    
      viewHistory();            
   }
}        
    
void QSudokuWindow::closeEvent(QCloseEvent * event)
{
    if (maybeSave()) {
        writeSettings();
        event->accept();
        delete sudoku_engine;
    } else {
        event->ignore();
    }
};

void QSudokuWindow::newFile()
{
    if (maybeSave()) {
        sudoku_widget->setSmallSquareSize(sudoku_widget->getSmallSquareSize());
        ChangeSolvedMode(false);
        setCurrentFile("");
    }
}

void QSudokuWindow::newFile2()
{
    if (maybeSave()) {
        sudoku_widget->setSmallSquareSize(2);
        ChangeSolvedMode(false);
        setCurrentFile("");
    }
}

void QSudokuWindow::newFile3()
{
    if (maybeSave()) {
        sudoku_widget->setSmallSquareSize(3);
        ChangeSolvedMode(false);
        setCurrentFile("");
    }
}

void QSudokuWindow::newFile4()
{
    if (maybeSave()) {
        sudoku_widget->setSmallSquareSize(4);
        ChangeSolvedMode(false);
        setCurrentFile("");
    }
}

void QSudokuWindow::newFile5()
{
    if (maybeSave()) {
        sudoku_widget->setSmallSquareSize(5);
        ChangeSolvedMode(false);
        setCurrentFile("");
    }
}


void QSudokuWindow::open()
{
   if (maybeSave())
   {
      QString fileName = QFileDialog::getOpenFileName( this, tr("Choose a file"), open_file_directory, tr("Text files (*.txt);;All files (*.*)"));
      if (!fileName.isEmpty())
      {
         loadFile(fileName);
         open_file_directory = QFileInfo(fileName).path();
      }
    }
}


void QSudokuWindow::edit()
{
   ChangeSolvedMode(false);
}

bool QSudokuWindow::save()
{
    if (curFile.isEmpty()) {
        return saveAs(false);
    } else {
        return saveFile(curFile, false);
    }
}

bool QSudokuWindow::save_with_solution()
{
    if (curFile.isEmpty()) {
        return saveAs(true);
    } else {
        return saveFile(curFile, true);
    }
}


bool QSudokuWindow::saveAs(bool with_solution)
{
   if (!curFile.isEmpty()) save_file_directory = QFileInfo(curFile).path();    
   
   QString fileName = QFileDialog::getSaveFileName( this, tr("Choose a file"), save_file_directory, tr("Text files (*.txt);;All files (*.*)"));
   
   if (fileName.isEmpty()) return false;
   if (QFileInfo(fileName).suffix().isEmpty()) fileName = fileName + ".txt";
   save_file_directory = QFileInfo(fileName).path();      
   return saveFile(fileName, with_solution);
}

void QSudokuWindow::about()
{
   QMessageBox::about(this, tr("About QSudoku"),
            tr("The <b>QSudoku</b> is application for "
               "generating and solve sudoku puzzles<br/>"
               "Copyright (C) 2005  Yulia Shabunio"));
}

void QSudokuWindow::documentWasModified()
{
    setWindowModified(true);
}


void QSudokuWindow::solve()
{
   try
   {   
      QApplication::setOverrideCursor(Qt::WaitCursor);
      maybeEdited();
      
      sudoku_engine->GetInformation( solution_count,hardness,variants_count, count_of_steps, count_of_choices, excess_items, solution_history); 
      QString information;
      if (solution_count == 0)
      {
         information = tr("<B>This puzzle can not be solved</B>");
         statusBar()->showMessage(tr("This puzzle has not solutions"), 2000);         
         show1Act->setEnabled(false);        
         show2Act->setEnabled(false);          
      }
      else
      {
         if (solution_count == 2)
         {
           information = tr("<B>This puzzle has more than one solution</B>");
           statusBar()->showMessage(tr("This puzzle has more than one solution"), 2000);         
           show1Act->setEnabled(true);        
           show2Act->setEnabled(true);              
           showFirstSolution();
         }
         else
         {
            if (solution_count == -1)
            {
               information = tr("<B>CAN NOT BE SOLVED - NEED TOO MANY STEPS</B>");           
            }else{                      
               information = tr("<B>SOLVED</B>");           
               show1Act->setEnabled(true);        
               show2Act->setEnabled(false);              
               showFirstSolution();               
            }
            information +=  QString(tr(" difficulty level = %1")).arg(hardness==1?"Easy":(hardness==2?"Medium":"Hard"));
            information +=  QString(tr(" variants_count = %1")).arg(variants_count);
            information +=  QString(tr(" count_of_steps = %1")).arg(count_of_steps);           
            information +=  QString(tr(" count_of_choices = %1")).arg(count_of_choices);                      
            statusBar()->showMessage(tr("Puzzle solved"), 2000);         
         }
      }
      description->setText(information);      
      ChangeSolvedMode(true);

      QApplication::restoreOverrideCursor();
   }catch(SudokuException e)
   {
      QApplication::restoreOverrideCursor();      
      QMessageBox::critical(this, tr("QSudoku: Error solving puzzle"), tr(e.Description.c_str()));
      statusBar()->showMessage(tr(e.Description.c_str()), 2000);      
   }
}

void QSudokuWindow::createActions()
{
    newAct = new QAction(QIcon(":/images/new.png"), tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

    new2Act = new QAction(QIcon(":/images/new2.png"), tr("&New 2x2x2x2"), this);
    new2Act->setShortcut(tr("Alt+2"));
    new2Act->setStatusTip(tr("Create a new file 2x2x2x2"));
    connect(new2Act, SIGNAL(triggered()), this, SLOT(newFile2()));

    new3Act = new QAction(QIcon(":/images/new3.png"), tr("&New 3x3x3x3"), this);
    new3Act->setShortcut(tr("Alt+3"));
    new3Act->setStatusTip(tr("Create a new file 3x3x3x3"));
    connect(new3Act, SIGNAL(triggered()), this, SLOT(newFile3()));

    new4Act = new QAction(QIcon(":/images/new4.png"), tr("&New 4x4x4x4"), this);
    new4Act->setShortcut(tr("Alt+4"));
    new4Act->setStatusTip(tr("Create a new file 4x4x4x4"));
    connect(new4Act, SIGNAL(triggered()), this, SLOT(newFile4()));

    new5Act = new QAction(QIcon(":/images/new5.png"), tr("&New 5x5x5x5"), this);
    new5Act->setShortcut(tr("Alt+5"));
    new5Act->setStatusTip(tr("Create a new file 5x5x5x5"));
    connect(new5Act, SIGNAL(triggered()), this, SLOT(newFile5()));
    
    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the puzzle to disk"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), this);
    saveAsAct->setStatusTip(tr("Save the puzzle under a new name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    saveWithSolutionAct = new QAction(tr("Save &with solution"), this);
    saveWithSolutionAct->setStatusTip(tr("Save the puzzle with its solution"));
    connect(saveWithSolutionAct, SIGNAL(triggered()), this, SLOT(save_with_solution()));

    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    
    solveAct = new QAction(QIcon(":/images/solve.png"), tr("Solve &puzzle"), this);
    solveAct->setStatusTip(tr("Solve puzzle"));
    solveAct->setShortcut(tr("F9"));    
    connect(solveAct, SIGNAL(triggered()), this, SLOT(solve()));

    editAct = new QAction(QIcon(":/images/edit.png"), tr("&Edit puzzle"), this);
    editAct->setStatusTip(tr("Edit puzzle"));
    editAct->setShortcut(tr("F2"));    
    connect(editAct, SIGNAL(triggered()), this, SLOT(edit()));
    
    show1Act = new QAction(QIcon(":/images/show1.png"), tr("Show &first solution"), this);
    show1Act->setStatusTip(tr("Show first solution"));
    connect(show1Act, SIGNAL(triggered()), this, SLOT(showFirstSolution()));           
    
    show2Act = new QAction(QIcon(":/images/show2.png"), tr("Show &second solution"), this);
    show2Act->setStatusTip(tr("Show second solution"));
    connect(show2Act, SIGNAL(triggered()), this, SLOT(showSecondSolution()));           

    gennewAct = new QAction(QIcon(":/images/gennew.png"), tr("&Generate new puzzle"), this);
    gennewAct->setStatusTip(tr("Generate new puzzle"));
    connect(gennewAct, SIGNAL(triggered()), this, SLOT(generateNew()));        
    
    genbaseAct = new QAction(QIcon(":/images/genbase.png"), tr("Generate puzzle on this &base"), this);
    genbaseAct->setStatusTip(tr("Generate puzzle on this base"));
    connect(genbaseAct, SIGNAL(triggered()), this, SLOT(generateOnBase()));        

    showHistoryAct = new QAction(QIcon(":/images/history.png"), tr("&View solve history"), this);
    showHistoryAct->setStatusTip(tr("View solve history"));
    showHistoryAct->setShortcut(tr("F3"));    
    showHistoryAct->setCheckable ( true );
    showHistoryAct->setChecked ( false );    
    connect(showHistoryAct, SIGNAL(triggered()), this, SLOT(viewHistory()));
    
    firstAct = new QAction(QIcon(":/images/first.png"), tr("View &initial state"), this);
    firstAct->setStatusTip(tr("View initial state"));
    connect(firstAct, SIGNAL(triggered()), this, SLOT(historyFirst()));

    lastAct = new QAction(QIcon(":/images/last.png"), tr("View &last state"), this);
    lastAct->setStatusTip(tr("View last state"));
    connect(lastAct, SIGNAL(triggered()), this, SLOT(historyLast()));

    nextAct = new QAction(QIcon(":/images/next.png"), tr("Step &next"), this);
    nextAct->setStatusTip(tr("Step next"));
    nextAct->setShortcut(Qt::CTRL+Qt::Key_Right);        
    connect(nextAct, SIGNAL(triggered()), this, SLOT(historyNext()));

    prevAct = new QAction(QIcon(":/images/prev.png"), tr("Step &previous"), this);
    prevAct->setStatusTip(tr("Step &previous"));
    prevAct->setShortcut(Qt::CTRL+Qt::Key_Left);        
    connect(prevAct, SIGNAL(triggered()), this, SLOT(historyPrev()));
    
    next2qAct = new QAction(QIcon(":/images/next2q.png"), tr("Step to next &questionable item"), this);
    next2qAct->setStatusTip(tr("Step to next questionable item"));
    connect(next2qAct, SIGNAL(triggered()), this, SLOT(historyNextToQuestion()));
    
    next2aAct = new QAction(QIcon(":/images/next2a.png"), tr("Step to next strikeout"), this);
    next2aAct->setStatusTip(tr("Step to next strikeout"));
    connect(next2aAct, SIGNAL(triggered()), this, SLOT(historyNextToAbort()));
    
    easyAct = new QAction(QIcon(":/images/easy.png"), tr("&Easy"), this);
    easyAct->setStatusTip(tr("Generate easy puzzle"));
    easyAct->setCheckable ( true );

    mediumAct = new QAction(QIcon(":/images/medium.png"), tr("&Medium"), this);
    mediumAct->setStatusTip(tr("Try to generate puzzle with medium difficulty"));
    mediumAct->setCheckable ( true );
    
    hardAct = new QAction(QIcon(":/images/hard.png"), tr("&Hard"), this);
    hardAct->setStatusTip(tr("Try to generate hard puzzle"));
    hardAct->setCheckable ( true );
    
    difficultyActGroup = new QActionGroup(this);
    difficultyActGroup->addAction(easyAct);
    difficultyActGroup->addAction(mediumAct);    
    difficultyActGroup->addAction(hardAct);        
    easyAct->setChecked ( true );    
    
}

void QSudokuWindow::createMenus()
{
   fileMenu = menuBar()->addMenu(tr("&File"));
   fileMenu->addAction(newAct);
   newSubMenu = fileMenu->addMenu(tr("New Si&ze"));
   
   newSubMenu->addAction(new2Act);
   newSubMenu->addAction(new3Act);
   newSubMenu->addAction(new4Act);
   newSubMenu->addAction(new5Act);            
     
   fileMenu->addAction(openAct);
   fileMenu->addAction(saveAct);
   fileMenu->addAction(saveAsAct);
   fileMenu->addAction(saveWithSolutionAct);    
   fileMenu->addSeparator();
   fileMenu->addAction(exitAct);
   
   solveMenu = menuBar()->addMenu(tr("&Sudoku"));
   solveMenu->addAction(solveAct);
   historyMenu = solveMenu->addMenu(tr("Solve history"));
   historyMenu->addAction(showHistoryAct);
   historyMenu->addAction(firstAct);
   //    historyMenu->addAction(prevAct);                    
   historyMenu->addAction(nextAct);
   historyMenu->addAction(next2qAct);            
   historyMenu->addAction(next2aAct);    
   historyMenu->addAction(lastAct);    
   
   solveMenu->addSeparator();       
   solveMenu->addAction(show1Act);    
   solveMenu->addAction(show2Act);     
   solveMenu->addSeparator();   
   solveMenu->addAction(editAct);    
   solveMenu->addSeparator();       
   solveMenu->addAction(gennewAct);    
   solveMenu->addAction(genbaseAct);     
   difficultyMenu = solveMenu->addMenu(tr("Set difficulty level"));
   difficultyMenu->addAction(easyAct);
   difficultyMenu->addAction(mediumAct);    
   difficultyMenu->addAction(hardAct);        
   
   menuBar()->addSeparator();
   
   helpMenu = menuBar()->addMenu(tr("&Help"));
   helpMenu->addAction(aboutAct);
   helpMenu->addAction(aboutQtAct);
}

void QSudokuWindow::createToolBars()
{
   fileToolBar = addToolBar(tr("File"));
   fileToolBar->addAction(newAct);
   fileToolBar->addAction(openAct);
   fileToolBar->addAction(saveAct);
    
   solveToolBar = addToolBar(tr("Sudoku"));
   solveToolBar->addAction(solveAct);
   solveToolBar->addSeparator();
   solveToolBar->addAction(show1Act);    
   solveToolBar->addAction(show2Act);        
   solveToolBar->addSeparator();
   solveToolBar->addAction(editAct);    
   solveToolBar->addSeparator();
   solveToolBar->addAction(gennewAct);    
   solveToolBar->addAction(genbaseAct);     
   solveToolBar->addSeparator();   
   solveToolBar->addAction(easyAct);
   solveToolBar->addAction(mediumAct);    
   solveToolBar->addAction(hardAct);           
    
   viewHistoryBar = addToolBar(tr("History"));
   viewHistoryBar->addAction(showHistoryAct);
   viewHistoryBar->addSeparator();   
   viewHistoryBar->addAction(firstAct);
//   viewHistoryBar->addAction(prevAct);            
   viewHistoryBar->addAction(nextAct);
   viewHistoryBar->addAction(next2qAct);           
   viewHistoryBar->addAction(next2aAct);    
   viewHistoryBar->addAction(lastAct);
   
}

void QSudokuWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void QSudokuWindow::readSettings()
{
    QSettings settings("Shabunio", "QSudoku");
    QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
    QSize size = settings.value("size", QSize(400, 400)).toSize();
    open_file_directory = settings.value("current_open_dir", QDir::currentPath()).toString();    
    save_file_directory = settings.value("current_save_dir", QDir::currentPath()).toString();    
    resize(size);
    move(pos);
}

void QSudokuWindow::writeSettings()
{
    QSettings settings("Shabunio", "QSudoku");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
    settings.setValue("current_open_dir", open_file_directory);
    settings.setValue("current_save_dir", save_file_directory);
}


bool QSudokuWindow::maybeSave()
{
    if (isWindowModified()) {
        int ret = QMessageBox::warning(this, tr("QSudoku"),
                     tr("The puzzle has been modified.\n"
                        "Do you want to save your changes?"),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;
};

void QSudokuWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    setWindowModified(false);
};

void QSudokuWindow::loadFile(const QString &fileName)
{
   try
   {
      QApplication::setOverrideCursor(Qt::WaitCursor);
      sudoku_widget->setUpdatesEnabled(false);
      sudoku_engine->LoadSudoku(fileName.toLocal8Bit().data());
      const SudokuTask & task = sudoku_engine->GetCurrentTask();
      sudoku_widget->setSmallSquareSize(task.SmallSize());
      for (int i = 0; i< task.Size(); i++)for (int j = 0; j< task.Size(); j++)
      {
         int int_val = task.Get(i, j);
         if (int_val > 0)
         {
            QChar c = Sudoku::GetCharByValue(int_val);
            sudoku_widget->SetInitialValue(i, j, c );
         }
      }

      
      ChangeSolvedMode(false);
      sudoku_widget->setUpdatesEnabled(true);    

      QApplication::restoreOverrideCursor();

      setCurrentFile(fileName);
      statusBar()->showMessage(tr("File loaded"), 2000);
   }catch(SudokuException e)
   {
      QApplication::restoreOverrideCursor();      
      sudoku_widget->setUpdatesEnabled(true);                
      QMessageBox::critical(this, "QSudoku: Error loading file", tr(e.Description.c_str()));
      statusBar()->showMessage(tr(e.Description.c_str()), 2000);      
   }
}

bool QSudokuWindow::saveFile(const QString &fileName, bool withSolution)
{
   try
   {   
      QApplication::setOverrideCursor(Qt::WaitCursor);
      
      maybeEdited();
      
      sudoku_engine->SaveSudoku(fileName.toLocal8Bit().data(), withSolution); 
      QApplication::restoreOverrideCursor();

      setCurrentFile(fileName);
      statusBar()->showMessage(tr("File saved"), 2000);
      return true;
   }catch(SudokuException e)
   {
      QApplication::restoreOverrideCursor();      
      QMessageBox::critical(this, "QSudoku: Error saving file", tr(e.Description.c_str()));
      statusBar()->showMessage(tr(e.Description.c_str()), 2000);      
      return false;
   }
}

void QSudokuWindow::maybeEdited()
{
   if ( sudoku_widget->GetEditMode()){
     SudokuTask task(sudoku_widget->getSmallSquareSize());
     for (int i = 0; i< task.Size(); i++)for (int j = 0; j< task.Size(); j++)
      {
         const QChar c = sudoku_widget->GetValue( i, j );
         if (c !=' '){
            task.Set(i, j, Sudoku::GetValueByChar(c.cell()));            
         }
      }       
      sudoku_engine->SetCurrentTask(task);
   }
}

void QSudokuWindow::showFirstSolution()
{
   if (solution_count == 0 ) return;
   SudokuDesk & desk = sudoku_engine->GetSolution(1);
   ShowSolution ( desk );  
   show1Act->setEnabled(false);   
   if (solution_count == 2 ) show2Act->setEnabled(true);                    
   std::list<SudokuChoice>::iterator ex_it;
   for(ex_it=excess_items.begin(); ex_it!=excess_items.end();ex_it++)
   {
      sudoku_widget->UpdateValue(ex_it->i,  ex_it->j, Sudoku::GetCharByValue(ex_it->value), QSudokuWidget::initial_but_exceeded);
   }
}

void QSudokuWindow::showSecondSolution()
{
   if (solution_count != 2 ) return;
   SudokuDesk & desk = sudoku_engine->GetSolution(2);   
   ShowSolution ( desk );  
   show2Act->setEnabled(false);                 
   show1Act->setEnabled(true);            
}

void QSudokuWindow::generateNew()
{
   if (!maybeSave()) return;
   try
   {
      QApplication::setOverrideCursor(Qt::WaitCursor);
      sudoku_widget->setUpdatesEnabled(false);
      
      SudokuTask prev_task(sudoku_widget->getSmallSquareSize());
      sudoku_engine->SetCurrentTask(prev_task);
      
      int d_level = 0;
      if (mediumAct->isChecked()) d_level = 1;
      if (hardAct->isChecked()) d_level = 2;      
      
      sudoku_engine->Generate(d_level);      
      
      const SudokuTask & task = sudoku_engine->GetCurrentTask();
      sudoku_widget->setSmallSquareSize(task.SmallSize());
      for (int i = 0; i< task.Size(); i++)for (int j = 0; j< task.Size(); j++)
      {
         int int_val = task.Get(i, j);
         if (int_val > 0)
         {
            QChar c = Sudoku::GetCharByValue(int_val);
            sudoku_widget->SetInitialValue(i, j, c );
         }
      }
      
      sudoku_engine->GetInformation( solution_count,hardness,variants_count, count_of_steps, count_of_choices, excess_items, solution_history); 
      QString information = QString(tr("<B>Generated</B> difficulty level = %1")).arg(hardness==1?"Easy":(hardness==2?"Medium":"Hard")); 
      
      ChangeSolvedMode(false);
      sudoku_widget->setUpdatesEnabled(true);    
      description->setText(information);

      QApplication::restoreOverrideCursor();

      setCurrentFile("");
      statusBar()->showMessage(tr("Sudoku generated!"), 2000);
   }catch(SudokuException e)
   {
      QApplication::restoreOverrideCursor();      
      sudoku_widget->setUpdatesEnabled(true);                
      QMessageBox::critical(this, "QSudoku: Error generating sudoku", tr(e.Description.c_str()));
      statusBar()->showMessage(tr(e.Description.c_str()), 2000);      
   }   
}

void QSudokuWindow::generateOnBase()
{
   try
   {
      QApplication::setOverrideCursor(Qt::WaitCursor);
      sudoku_widget->setUpdatesEnabled(false);
      
      maybeEdited();
      
      int d_level = 0;
      if (mediumAct->isChecked()) d_level = 1;
      if (hardAct->isChecked()) d_level = 2;            
      sudoku_engine->Generate(d_level);      
      
      const SudokuTask & task = sudoku_engine->GetCurrentTask();
      sudoku_widget->setSmallSquareSize(task.SmallSize());
      for (int i = 0; i< task.Size(); i++)for (int j = 0; j< task.Size(); j++)
      {
         int int_val = task.Get(i, j);
         if (int_val > 0)
         {
            QChar c = Sudoku::GetCharByValue(int_val);
            sudoku_widget->SetInitialValue(i, j, c );
         }
      }

      sudoku_engine->GetInformation( solution_count,hardness,variants_count, count_of_steps, count_of_choices, excess_items, solution_history); 
      QString information = QString(tr("<B>Generated</B> difficulty level = %1")).arg(hardness==1?"Easy":(hardness==2?"Medium":"Hard")); 
      
      ChangeSolvedMode(false);
      sudoku_widget->setUpdatesEnabled(true);    
      description->setText(information);

      QApplication::restoreOverrideCursor();

      statusBar()->showMessage(tr("Sudoku generated!"), 2000);
   }catch(SudokuException e)
   {
      QApplication::restoreOverrideCursor();      
      sudoku_widget->setUpdatesEnabled(true);                
      QMessageBox::critical(this, "QSudoku: Error generating sudoku", tr(e.Description.c_str()));
      statusBar()->showMessage(tr(e.Description.c_str()), 2000);      
   }      
}

void QSudokuWindow::ShowSolution(SudokuDesk & desk)
{
   for (int i = 0; i< desk.GetSize(); i++)for (int j = 0; j< desk.GetSize(); j++)
   {
      int int_val = desk.GetItem(i, j);
      if (int_val > 0)
      {
         QChar c = Sudoku::GetCharByValue(int_val);
         sudoku_widget->UpdateValue(i,  j, c, QSudokuWidget::solved);
      }
   }   
}

void QSudokuWindow::viewHistory()
{
   bool checked = showHistoryAct->isChecked(); 
   if (checked)
   {
      if (solution_count == 0 ){ showHistoryAct->setChecked(false); return; };
      firstAct->setEnabled(true);    
      lastAct->setEnabled(true);          
      nextAct->setEnabled(true);          
      next2qAct->setEnabled(true);                
      next2aAct->setEnabled(true);                
      prevAct->setEnabled(true);          
      sudoku_widget->SetViewHistoryMode(true);      
      historyFirst();      
      
   }else{
      firstAct->setEnabled(false);    
      lastAct->setEnabled(false);          
      nextAct->setEnabled(false);          
      next2qAct->setEnabled(false);                
      next2aAct->setEnabled(false);                
      prevAct->setEnabled(false);          
      sudoku_widget->SetViewHistoryMode(false);            
      if (solution_count == 0 ) return;
      historyLast(); 
   }  
};
void QSudokuWindow::historyFirst()
{
   if (solution_count == 0 ) return;   
   const SudokuTask & task = sudoku_engine->GetCurrentTask();
   
   for (int i = 0; i< task.Size(); i++)for (int j = 0; j< task.Size(); j++)
   {
      int int_val = task.Get(i, j);
      if (int_val == 0)
      {
         QChar c = Sudoku::GetCharByValue(int_val);
         sudoku_widget->UpdateValue(i,  j, ' ', QSudokuWidget::empty);
      }
   }     
   solution_history_iterator = solution_history.begin();   
   type_of_last_step = SudokuSolveObserver::unknown;
};
void QSudokuWindow::historyLast()
{
   if (solution_count == 0 ) return;   
   solution_history_iterator = solution_history.end();      
   showFirstSolution();
};

void QSudokuWindow::historyNext()
{
   if ( solution_count == 0 ) return;   

   if ( solution_history_iterator == solution_history.end() )
   {
      showFirstSolution();      
      return;
   }
   
   SudokuSolveStep prev_step(0, 0, 0, SudokuSolveObserver::exact);
   
   if ( solution_history_iterator != solution_history.begin() )
   {
      solution_history_iterator--;
      prev_step = *solution_history_iterator;
      solution_history_iterator++;
   }
   
   if (  prev_step.type == SudokuSolveObserver::impossible
      || prev_step.type == SudokuSolveObserver::impossible_column
      || prev_step.type == SudokuSolveObserver::impossible_row      
      || prev_step.type == SudokuSolveObserver::impossible_square
      || prev_step.type == SudokuSolveObserver::aborted      
      )
   {
      sudoku_widget->ResetWrongValues();
   }      
      
   SudokuSolveStep step = *solution_history_iterator;
   while ( step.type == SudokuSolveObserver::exact && step.value == Sudoku::GetValueByChar( sudoku_widget->GetValue( step.i, step.j ).cell()) )
   {
      solution_history_iterator++;  
      step = *solution_history_iterator;
      if ( solution_history_iterator == solution_history.end() )
      {
         showFirstSolution();
         return;
      }
   }
   
   while ( step.type == SudokuSolveObserver::unknown )
   {
      sudoku_widget->UpdateValue(step.i,  step.j,  QChar(' '), QSudokuWidget::empty);      
      solution_history_iterator++;  
      step = *solution_history_iterator;
      if ( solution_history_iterator == solution_history.end() )
      {
         historyLast();
         return;
      }
   }
   
   int i, j, size, small_size;
   small_size = sudoku_widget->getSmallSquareSize();
   size = small_size*small_size;
   switch ( step.type )
   {
      case SudokuSolveObserver::exact:
         sudoku_widget->UpdateValue(step.i,  step.j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::solved);
         break;
      case SudokuSolveObserver::questionable:
         sudoku_widget->UpdateValue(step.i,  step.j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::can_be);
         break;
      case SudokuSolveObserver::impossible:
         sudoku_widget->UpdateValue(step.i,  step.j, QChar('X'), QSudokuWidget::wrong);
         break;
      case SudokuSolveObserver::impossible_column:
         for (i=0; i<size; i++)
         {
            if ( sudoku_widget->GetValue( i, step.j ) == QChar(' ') ) 
            {
               sudoku_widget->UpdateValue(i,  step.j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::wrong);
            }
         }
         break;
      case SudokuSolveObserver::impossible_row:
         for (j=0; j<size; j++)
         {
            if ( sudoku_widget->GetValue( step.i, j ) == QChar(' ') ) 
            {
               sudoku_widget->UpdateValue(step.i,  j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::wrong);
            }
         }         
         break;
      case SudokuSolveObserver::impossible_square:
         for (i=step.i-step.i%small_size; i<(step.i-step.i%small_size+small_size); i++)
         for (j=step.j-step.j%small_size; j<(step.j-step.j%small_size+small_size); j++)         
         {
            if ( sudoku_widget->GetValue( i, j ) == QChar(' ') ) 
            {
               sudoku_widget->UpdateValue(i,  j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::wrong);
            }
         }         
         break;
      case SudokuSolveObserver::aborted:         
         sudoku_widget->UpdateValue(step.i,  step.j,  Sudoku::GetCharByValue(step.value), QSudokuWidget::aborted);
         break;
      case SudokuSolveObserver::unknown:
         sudoku_widget->UpdateValue(step.i,  step.j,  QChar(' '), QSudokuWidget::empty);
         break;
   }
   type_of_last_step = step.type;
   solution_history_iterator++;   
};        

void QSudokuWindow::historyPrev()
{
  
};

void QSudokuWindow::historyNextToQuestion()
{
   if (solution_count == 0 ) return;   
   sudoku_widget->setUpdatesEnabled(false);    
   historyNext();  
   while ( solution_history_iterator != solution_history.end() 
         && type_of_last_step != SudokuSolveObserver::questionable 
         && type_of_last_step != SudokuSolveObserver::impossible 
         && type_of_last_step != SudokuSolveObserver::impossible_column 
         && type_of_last_step != SudokuSolveObserver::impossible_row 
         && type_of_last_step != SudokuSolveObserver::impossible_square                            
         )
   {
      historyNext();  
   }
   sudoku_widget->setUpdatesEnabled(true);    
}
void QSudokuWindow::historyNextToAbort()
{
   if (solution_count == 0 ) return;   
   sudoku_widget->setUpdatesEnabled(false);    
   historyNext();  
   while ( solution_history_iterator != solution_history.end() 
         && type_of_last_step != SudokuSolveObserver::impossible 
         && type_of_last_step != SudokuSolveObserver::impossible_column 
         && type_of_last_step != SudokuSolveObserver::impossible_row 
         && type_of_last_step != SudokuSolveObserver::impossible_square                            
         )
   {
      historyNext();  
   }
   sudoku_widget->setUpdatesEnabled(true);       
}

