/****************************************************************************

    QSudoku - a application for solving and generating sudoku puzzles
    Copyright (C) 2005  Yulia Shabunio

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty or
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

****************************************************************************/

#ifndef QSUDOKU
#define QSUDOKU 1
#include <QMainWindow>
#include <QApplication>
#include "QSudokuWidget.h"
#include "SudokuEngine/Sudoku.h"

class QSudokuWindow : public QMainWindow
{
	Q_OBJECT
private slots:
    void newFile();
    void newFile2();    
    void newFile3();    
    void newFile4();        
    void newFile5();        
    void open();
    bool save();
    bool save_with_solution();    
    bool saveAs(bool with_solution = false);
    void about();
    void documentWasModified();
    void solve();    
    void edit();       
    void showFirstSolution();    
    void showSecondSolution();         
    void generateNew();    
    void generateOnBase();         

    void viewHistory();
    void historyFirst();
    void historyLast();
    void historyNext();        
    void historyNextToQuestion();            
    void historyNextToAbort();                
    void historyPrev();    
    
private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    bool maybeSave();
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName, bool withSolution);
    void setCurrentFile(const QString &fileName);
    void maybeEdited();    
    void ChangeSolvedMode(bool is_solved);      
    void ShowSolution(SudokuDesk & desk);      

public:
	QSudokuWindow(QWidget * parent = 0, Qt::WFlags flags = 0);
	
protected:
  virtual void closeEvent(QCloseEvent * event);

   QString curFile;

   QSudokuWidget * sudoku_widget;
   Sudoku * sudoku_engine;
   QLabel * description;   

   QMenu *fileMenu;
   QMenu *helpMenu;
   QMenu *newSubMenu;
   QMenu *solveMenu;
   QMenu *historyMenu;   
   QMenu *difficultyMenu;   
   
   QToolBar *fileToolBar;
   QToolBar *solveToolBar;   
   QToolBar *viewHistoryBar;      
   QAction *newAct;                    
   QAction *new2Act;                
   QAction *new3Act;
   QAction *new4Act;    
   QAction *new5Act;        
   QAction *openAct;
   QAction *saveAct;
   QAction *saveAsAct;
   QAction *exitAct;   
   QAction *aboutAct;
   QAction *aboutQtAct;
   QAction *solveAct;   
   QAction *saveWithSolutionAct;
   QAction *editAct;   
   QAction *show1Act;      
   QAction *show2Act;         
   QAction *gennewAct;      
   QAction *genbaseAct;        
   
   QAction *showHistoryAct;      
   QAction *firstAct;         
   QAction *lastAct;      
   QAction *nextAct;        
   QAction *next2qAct;           
   QAction *next2aAct;              
   QAction *prevAct;           

   QAction *easyAct;           
   QAction *hardAct;              
   QAction *mediumAct;         
   QActionGroup * difficultyActGroup;

   QString open_file_directory;
   QString save_file_directory;   
   
   std::list<SudokuChoice> excess_items;
   std::list<SudokuSolveStep> solution_history;   
   std::list<SudokuSolveStep>::const_iterator solution_history_iterator;
   int solution_count, hardness, count_of_steps, count_of_choices;
   double variants_count;   
   SudokuSolveObserver::SudokuStepType type_of_last_step;
};

#endif
