#include "QSudokuWidget.h"
//#include <QPainter>
//#include <QMessageBox>
//#include <QFontMetrics>
#include <math.h>

QSudokuWidget::QSudokuWidget(QWidget * parent, Qt::WFlags f):QWidget(parent, f )
{
   QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Expanding);
   policy.setHeightForWidth(true);
   setSizePolicy(policy);
   setFocusPolicy (Qt::StrongFocus);
   setBackgroundRole(QPalette::NoRole);
   inner_line_width = 1;
   outer_line_width = 3;        
   item_size = 10;   

   small_square_size = 3;
   items = new QSudokuVisibleItem[small_square_size*small_square_size*small_square_size*small_square_size];

   square = item_size*small_square_size*small_square_size + (small_square_size + 1)*outer_line_width + small_square_size*(small_square_size-1)*inner_line_width;
   setMinimumSize ( square, square );

   current_x = 0;
   current_y = 0;
   
   pixmap = NULL;
   pixmap_painter = NULL;

   QFont new_font(font());
   new_font.setBold(true);
   setFont(new_font);   
   
   edit_mode = true;   
};

QSudokuWidget::~QSudokuWidget()
{
   delete pixmap_painter;
   delete pixmap;
   delete[] items;
}
;
	
void QSudokuWidget::resizeEvent ( QResizeEvent * event )
{
   int min_dimension = height(); if ( min_dimension > width() ) min_dimension = width();
   item_size = ( min_dimension - (small_square_size + 1)*outer_line_width - small_square_size*(small_square_size-1)*inner_line_width )/(small_square_size*small_square_size);
   square = item_size*small_square_size*small_square_size + (small_square_size + 1)*outer_line_width + small_square_size*(small_square_size-1)*inner_line_width;
   x0 = width()/2 - square/2;
   y0 = height()/2- square/2;
   big_shift = item_size*small_square_size+inner_line_width*(small_square_size-1)+outer_line_width;
   small_shift = item_size+inner_line_width;   
   
   QFontMetrics fm = fontMetrics();
   int fm_h = fm.height();
   int char_size = fm_h; // if ( char_size < fm_w) char_size = fm_w; 
   
   float size_cf = 1.0*item_size/char_size;
   if (fabs(size_cf -1 ) >= 0.2 ){

      int pixelSize = font().pixelSize();
      int pointSize = font().pointSize();   
      
      QFont new_font( font() );
      new_font.setBold(true);
      if (pixelSize == -1)
      {
         new_font.setPointSize(int(pointSize*size_cf));
      }else
      {
         new_font.setPixelSize(int(pixelSize*size_cf));      
      }
      setFont(new_font);   
   }

   delete pixmap_painter;   
   delete pixmap;
   pixmap = new QPixmap( width(), height() );
   pixmap->fill(this, 0, 0 );
   pixmap_painter = new QPainter( pixmap );
   
   pixmap_painter->setRenderHint(QPainter::TextAntialiasing, true );
   pixmap_painter->setRenderHint(QPainter::Antialiasing, false );
   pixmap_painter->setFont(font());     
  
   QRect r( x0, y0, square, square);
   pixmap_painter->setPen(Qt::NoPen);   
   pixmap_painter->setBrush( palette().base());
   pixmap_painter->drawRect( r ) ;
   pixmap_painter->setBrush( palette().foreground());

   int i, j; QRectF text_rect;

   for(i = 0; i< small_square_size*small_square_size; i++ )for(j = 0; j< small_square_size*small_square_size; j++ )
   {
      UpdatePixmap( i, j, false);
   }   

   pixmap_painter->setPen(Qt::NoPen);      
   //Grid
   for(i = 0; i<= small_square_size; i++ )
   {
      pixmap_painter->drawRect(x0, y0+i*big_shift, square, outer_line_width);
      pixmap_painter->drawRect(x0+i*big_shift, y0, outer_line_width, square);      
      
      if ( i< small_square_size)
         for( int j = 1; j < small_square_size; j ++ )
         {
            pixmap_painter->drawRect(x0, y0+i*big_shift+j*small_shift+outer_line_width-inner_line_width, square, inner_line_width);            
            pixmap_painter->drawRect(x0+i*big_shift+j*small_shift+outer_line_width-inner_line_width, y0, inner_line_width,square);                        
         }
   }
   
}

void QSudokuWidget::UpdatePixmap(int i, int j, bool need_erase)
{
   if (!pixmap_painter) return;
      
   QSudokuVisibleItem item = items[i*small_square_size*small_square_size+j];
   QRectF text_rect = GetItemRectangle( i, j );
   if (need_erase)
   {
      pixmap_painter->setPen(Qt::NoPen);      
      pixmap_painter->setBrush(palette().base());
      pixmap_painter->drawRect(text_rect);
   }

   QFont fnt = pixmap_painter->font();
   if (item.type == empty) return;
   switch(item.type)
   {
      case empty: return;
      case initial:
         fnt.setItalic(false);       
         fnt.setBold(true);                                    
         pixmap_painter->setPen(palette().color(QPalette::Text)); 
         break;
      case initial_but_exceeded:
         fnt.setItalic(false);
         fnt.setBold(true);         
         pixmap_painter->setPen(Qt::gray);           
         break;         
      case solved:
         fnt.setItalic(true);         
         fnt.setBold(false);                  
         pixmap_painter->setPen(Qt::blue);           
         break;         
      case can_be:
      case aborted:                         
         fnt.setItalic(true);  
         fnt.setBold(false);                           
         pixmap_painter->setPen(Qt::darkGreen);           
         break;         
      case wrong:                
         fnt.setItalic(false);
         fnt.setBold(false);                                    
         pixmap_painter->setPen(Qt::red);         
   }
   pixmap_painter->setFont(fnt);                       
   pixmap_painter->drawText(text_rect, QString(item.value), QTextOption(Qt::AlignHCenter | Qt::AlignVCenter) );                       
   
   if ( item.type == aborted )
   {
      pixmap_painter->setPen(Qt::darkGreen);  
      pixmap_painter->drawLine( text_rect.topLeft () , text_rect.bottomRight() );
      pixmap_painter->drawLine( text_rect.bottomLeft () , text_rect.topRight() );      
   }    
};
     	
QRectF QSudokuWidget::GetItemRectangle(int i, int j)
{
   
   int i1 = i / small_square_size, i2 = i % small_square_size;
   int j1 = j / small_square_size, j2 = j % small_square_size;   
   return QRectF( x0+j1*big_shift+j2*small_shift+outer_line_width
                , y0+i1*big_shift+i2*small_shift+outer_line_width
                , item_size
                , item_size 
                );   
}

void QSudokuWidget::paintEvent(QPaintEvent *)
{
     
   QPainter p(this);

   if (pixmap)
   {
       p.drawPixmap ( 0, 0, *pixmap );
   }

   QRectF text_rect;
   if (edit_mode && hasFocus())
   {
      text_rect = GetItemRectangle( current_x, current_y );
      p.setBrush(Qt::NoBrush);   
      p.setPen(QPen(Qt::blue, 2)); 
      p.drawRect(text_rect);
   }   

   if (view_history_mode)
   {
      text_rect = GetItemRectangle( current_x, current_y );
      p.setBrush(Qt::NoBrush);   
      p.setPen(QPen(Qt::red, 2)); 
      p.drawRect(text_rect);

      p.setPen(Qt::NoPen); 
      
      QColor r1(255, 0, 0, 25);
      QColor g1(0, 255, 0, 25);
      QColor b1(0, 0, 255, 25);
      
      QRectF selection_rect = text_rect;
      selection_rect.setLeft(x0);
      selection_rect.setWidth(square);      
      p.setBrush(r1);   
      p.drawRect(selection_rect);
      
      selection_rect = text_rect;
      selection_rect.setTop(y0);
      selection_rect.setHeight(square);            
      p.setBrush(g1);   
      p.drawRect(selection_rect);

      int x1 = current_x / small_square_size; 
      int y1 = current_y / small_square_size;       
      text_rect = GetItemRectangle( x1*small_square_size, y1*small_square_size );
      
      selection_rect = text_rect;
      selection_rect.setWidth(big_shift);
      selection_rect.setHeight(big_shift);            
      p.setBrush(b1);   
      p.drawRect(selection_rect);      
   }   

   p.end();
}

void QSudokuWidget::keyPressEvent ( QKeyEvent * e )
{
   if (edit_mode && current_y >= 0 && current_y < small_square_size*small_square_size
                 && current_x >= 0 && current_x < small_square_size*small_square_size)
   {
      QSudokuVisibleItem * item = items + current_x*small_square_size*small_square_size+current_y;      
      switch ( e->key() )
      {
         case Qt::Key_Enter: 
         case Qt::Key_Return:             
            if (current_x < small_square_size*small_square_size-1){ current_x++; update(); };
            return;
         case Qt::Key_Home: 
            current_y = 0;
            update();
            return;
         case Qt::Key_End:             
            current_y = small_square_size*small_square_size-1;
            update();
            return;
         case Qt::Key_PageUp:
            current_x = 0;
            update();
            return;
         case Qt::Key_PageDown:            
            current_x = small_square_size*small_square_size-1;
            update();
            return;
         case Qt::Key_Left:             
            if ( current_y > 0 ){ current_y--; update(); };
            return;
         case Qt::Key_Right:            
            if ( current_y < small_square_size*small_square_size-1 ){ current_y++; update(); };
            return;
         case Qt::Key_Up:            
            if ( current_x > 0 ){ current_x--; update(); };
            return;
         case Qt::Key_Down:           
            if ( current_x < small_square_size*small_square_size-1 ){ current_x++; update(); };
            return;
         case Qt::Key_Tab:           
            if ( current_y < small_square_size*small_square_size-1 )
            { 
               current_y++; 
               update(); 
            }else
            {
               if ( current_x < small_square_size*small_square_size-1 )
               { 
                  current_x++; 
                  current_y = 0;
                  update(); 
               }
            }
            return;
         case Qt::Key_Backtab:           
            if ( current_y > 0 )
            { 
               current_y--; 
               update(); 
            }else
            {
               if ( current_x > 0 )
               { 
                  current_x--; 
                  current_y = small_square_size*small_square_size-1;
                  update(); 
               }
           }   
            return;
         case Qt::Key_Backspace:           
            item->type = empty;
            item->value = ' '; 
            UpdatePixmap(current_x, current_y, true);
            if ( current_y > 0 )
            { 
               current_y--; 
            }else
            {
               if ( current_x > 0 )
               { 
                  current_x--; 
                  current_y = small_square_size*small_square_size-1;
               }
            }
            update();             
            emit contentsChanged();
            return;            
         case Qt::Key_Delete:           
            item->type = empty;
            item->value = ' '; 
            UpdatePixmap(current_x, current_y, true);
            update(); 
            emit contentsChanged();
            return;                     
      };
            
      if (e->text().isEmpty()) return;
      
      QChar c = e->text().at(0);
      
      if (c.row() != 0) return;
      
      if ( c == ' ' || c == '0' )
      { 
         item->type = empty; 
         item->value = ' '; 
         UpdatePixmap(current_x, current_y, true); 
         if ( current_y < small_square_size*small_square_size-1 )
         { 
            current_y++; 
         }else
         {
            if ( current_x < small_square_size*small_square_size-1 )
            { 
               current_x++; 
               current_y = 0;
            }
         }
         update(); 
         emit contentsChanged();
         return; 
      };
      
      int int_value = Sudoku::GetValueByChar(c.cell());
      if ( int_value >= 1 && int_value <= small_square_size*small_square_size )
      {
         item->type = initial; 
         item->value = c.toUpper(); 
         UpdatePixmap(current_x, current_y, true);
         if ( current_y < small_square_size*small_square_size-1 )
         { 
            current_y++; 
         }else
         {
            if ( current_x < small_square_size*small_square_size-1 )
            { 
               current_x++; 
               current_y = 0;
            }
         }
         update();
         emit contentsChanged();         
         return;
      }
   }
};

void QSudokuWidget::SetInitialValue( const int i, const int j, const QChar value)
{
   QSudokuVisibleItem * item = items + i*small_square_size*small_square_size+j;         
   item->type = initial; 
   item->value = value; 
   UpdatePixmap(i, j, true); 
   update();
   emit contentsChanged();   
};

void QSudokuWidget::UpdateValue( const int i, const int j, const QChar value, QSudokuVisibleItemType type)
{
   QSudokuVisibleItem * item = items + i*small_square_size*small_square_size+j;         
   if ( item->type == initial_but_exceeded ) return;   
   if ( item->type == initial && type != initial_but_exceeded ) return;
   item->type = type; 
   item->value = value; 
   if ( type == empty ) item->value = 0;
   if (!edit_mode)
   {
      current_x = i;
      current_y = j;      
   }
   UpdatePixmap(i, j, true); 
   update();
};

void QSudokuWidget::ResetWrongValues()
{
   QSudokuVisibleItem * item;
   for(int i=0;i<small_square_size*small_square_size;i++)
   for(int j=0;j<small_square_size*small_square_size;j++)   
   {
      item= items + i*small_square_size*small_square_size+j;               
      if (item->type == wrong || item->type == aborted )
      {
         item->type = empty;
         item->value = ' '; 
         UpdatePixmap(i, j, true);
      }
   }
   update();   
}
	
QChar QSudokuWidget::GetValue( const int i, const int j) const
{
   QSudokuVisibleItem * item = items + i*small_square_size*small_square_size+j;         
   if (item->type == empty ) return QChar(' ');
   return item->value; 
}
	
void QSudokuWidget::mousePressEvent ( QMouseEvent * e )
{
   if (!edit_mode) return;
   QRect all( x0, y0, square, square);
   if (!all.contains( e->pos(), true)) return;
   current_y = (e->x()-x0)*small_square_size*small_square_size/square;
   current_x = (e->y()-y0)*small_square_size*small_square_size/square; 
   update();
};

int QSudokuWidget::getSmallSquareSize()
{
   return small_square_size;
};

void QSudokuWidget::setSmallSquareSize(int new_small_square_size)
{

   delete pixmap_painter;
   delete pixmap;
   delete[] items;

   small_square_size = new_small_square_size;
   items = new QSudokuVisibleItem[small_square_size*small_square_size*small_square_size*small_square_size];

   item_size = 10;      
   square = item_size*small_square_size*small_square_size + (small_square_size + 1)*outer_line_width + small_square_size*(small_square_size-1)*inner_line_width;
   current_x = 0;
   current_y = 0;
   
   pixmap = NULL;
   pixmap_painter = NULL;
   
   setMinimumSize ( square, square );

   updateGeometry();
   resizeEvent( NULL );
   update();   
   emit contentsChanged();   
};
