/****************************************************************************

    QSudoku - a application for solving and generating sudoku puzzles
    Copyright (C) 2005  Yulia Shabunio

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

 ****************************************************************************/

#ifndef QSUDOKUWIDGET
#define QSUDOKUWIDGET 1

#include <QtGui>
#include "SudokuEngine/Sudoku.h"

class QSudokuWidget : public QWidget
{
  Q_OBJECT
public:
	QSudokuWidget(QWidget * parent = 0, Qt::WFlags f = 0);
	virtual ~QSudokuWidget();

   enum QSudokuVisibleItemType
   {
      empty
    , initial // Item was loadede from file our entered by hands
    , initial_but_exceeded // Exceeded element of sudoku
    , solved //Found by solver
    , can_be //Item cant be this value
    , wrong //Can not be where
    , aborted //Was aborted
   };
	
	int getSmallSquareSize();
	void setSmallSquareSize(const int new_small_square_size);
	
	void SetInitialValue( const int i, const int j, const QChar value);
	QChar GetValue( const int i, const int j) const;	
   void UpdateValue( const int i, const int j, const QChar value, QSudokuVisibleItemType type);	
	
	void SetEditMode(bool mode){edit_mode = mode; if(edit_mode) view_history_mode = false; update();};
	bool GetEditMode(){return edit_mode; };
	void SetViewHistoryMode(bool mode){view_history_mode = mode; if(view_history_mode) edit_mode = false; update();};
	bool GetViewHistoryMode(){return view_history_mode; };
	
	void ResetWrongValues();
signals:
   void contentsChanged();
   	
protected:
  virtual void paintEvent ( QPaintEvent * event );
  virtual void resizeEvent ( QResizeEvent * event );
  virtual void keyPressEvent ( QKeyEvent * e );
  virtual void mousePressEvent ( QMouseEvent * e );
private:
   

   struct QSudokuVisibleItem
   {
      QChar value;
      QSudokuVisibleItemType type;
      QSudokuVisibleItem():value(' '), type(empty){};
      QSudokuVisibleItem(QSudokuVisibleItem & i):value(i.value), type(i.type){};
   };
   
   int small_square_size;
   QSudokuVisibleItem * items;

   int current_x;
   int current_y;
   
   QPixmap * pixmap;
   QPainter * pixmap_painter;
   
/*   
   //highlight sudoku items
   int hl_row;
   int hl_column;
   int hl_square_x;
   int hl_square_y;
*/   
   //Modes
   bool edit_mode;
   bool view_history_mode;   
   
   //Draw temp variables
   int inner_line_width;
   int outer_line_width;        
   int item_size;
   int square;
   int x0;
   int y0;
   int big_shift;
   int small_shift;   

   QRectF GetItemRectangle(int i, int j);
   void UpdatePixmap(int i, int j, bool need_erase); 
   
};
#endif

