#include "Sudoku.h"
#include <fstream>
#include <math.h>
 
void Sudoku::LoadSudoku(const std::string filename)
{

   /* TODO (#1#):    .
                      , ,
                  . */
   std::ifstream * f;
   f = new std::ifstream(filename.c_str());
   if (!f->is_open()) throw SudokuException(std::string(QT_TR_NOOP("Can't open file: "))+filename);

   char c;
   while( f->get(c) && ( c == '\n' || c == '\r' || c == ' ' || c == '\t' ) ); //Skip empty lines;
   int counter = 1;
   
   while( f->get(c) && c != '\n' && c != '\r' ){
      if ( c == ' ' || c == '\t' ) continue;
      counter++;
   }
   f->close();
   delete f;

   if (  ( counter != 4 )
      && ( counter != 9 )
      && ( counter != 16 )
      && ( counter != 25 )
      )
   {
      std::string s = std::string(QT_TR_NOOP("Incorrect file format. First line of file has wrong count of nonspace symbols (must be 4, 9, 16 or 25)"));
      throw SudokuException( s );
   }
   int test_small_size = int( round( sqrt(counter) ) );

   SudokuTask test(test_small_size);

   f = new std::ifstream(filename.c_str());
   if (!f->is_open())
   {
      throw SudokuException(std::string("Can't open file:")+filename);
   }

   int i=0, j=0, value = 0;
   while( f->get(c) )
   {
      if ( c == ' ' || c == '\t' ) continue;
      if ( c == '\n' || c == '\r' )
      {
         if ( j == 0 ) continue; //Skip empty line
         i++;
         j=0;
         if (i >= counter) break; //Desk was successfully readed
         continue;
      };
      if ( j >= counter )
      {
         std::string s = QT_TR_NOOP("Incorrect file format. Too many nonspace symbols in line ");
         s+=i;
         throw SudokuException(s);
      }

      value = GetValueByChar(c);
      if ( value == 0 )
      {
         j++;
         continue;
      };

      if ( value > counter )
      {
         throw SudokuException(QT_TR_NOOP("Incorrect file format. Too big value for sudoku with size = ") + counter);
      }

      test.Set(i, j, value);
      j++;
   }

   current_task = test;
   solution_count = -1;
};

void Sudoku::SaveSudoku(const std::string filename, bool with_solution)
{
   int i, j;
   std::ofstream * f;
   f = new std::ofstream(filename.c_str());
   if (!f->is_open()) throw SudokuException(std::string(QT_TR_NOOP("Can't open file: "))+filename);
   for(i=0; i< current_task.Size(); i++)
   {
      for(j=0; j<current_task.Size(); j++)
      {
         (*f)<<GetCharByValue(current_task.Get(i, j));
      }
      (*f)<<'\n';
   }
   (*f)<<'\n'<<'\n';
   if (with_solution)
   {
      switch(solution_count)
      {
         case -1: (*f)<<"This task was not solved.\n"; break;
         case 0: (*f)<<"This task can not be solved.\n"; break;        
         case 2: 
            (*f)<<"This sudoku has more than one solution.\n\n";
            (*f)<<"SOLUTION.\n";
            for(i=0; i< current_task.Size(); i++)
            {
               for(j=0; j<current_task.Size(); j++)
               {
                  (*f)<<GetCharByValue(d2.GetItem(i, j));
               }
               (*f)<<'\n';
            }      
            (*f)<<'\n';
         case 1: 
            (*f)<<"SOLUTION.\n";
            for(i=0; i< current_task.Size(); i++)
            {
               for(j=0; j<current_task.Size(); j++)
               {
                  (*f)<<GetCharByValue(d1.GetItem(i, j));
               }
               (*f)<<'\n';
            }      
            break; 
      }       
   }
   delete f;
}

void Sudoku::Solve(){
   SudokuSolver ss(current_task);
   solution_count = ss.Solve();
   if (solution_count > 0) d1 = ss.GetFirstSolve();
   if (solution_count > 1) d2 = ss.GetSecondSolve();   
}

SudokuDesk & Sudoku::GetSolution(int number)
{
   if ( number > solution_count ) throw SudokuException(QT_TR_NOOP("This solution does not exists"));
   if ( number == 1 ) return d1;
   if ( number == 2 ) return d2;   
   throw SudokuException(QT_TR_NOOP("This solution unknown in this version"));   
}
   
void Sudoku::GetInformation( int & solution_count, int & hardness, double & variants_count, int & count_of_steps, int & count_of_choices, std::list<SudokuChoice> & excess_items, std::list<SudokuSolveStep> & solution_history)
{
   SudokuSolver ss(current_task);
   ss.SaveSolve();
   solution_count = ss.Solve();
   this->solution_count = solution_count;   
   if (solution_count > 0) d1 = ss.GetFirstSolve();
   if (solution_count > 1) d2 = ss.GetSecondSolve();   

   if (solution_count)
   {
      excess_items = ss.GetExcessItems();      
      solution_history = ss.GetSolveHistory();
      count_of_steps = solution_history.size();
      count_of_choices = ss.GetCountOfChoices();
      
      SudokuDesk test_desk(current_task.SmallSize());
      test_desk.DisableNormalization();
      int i, j;
      for(i=0;i<current_task.Size();i++) for(j=0;j<current_task.Size();j++)
      {
         if ( current_task.Get(i, j) > 0) test_desk.SetItem(i, j, current_task.Get(i, j));
      }      
      if (test_desk.IsSolved())
      {
         hardness = 1;
         variants_count = 1;
      }else
      {
         SudokuDesk test_desk_2(current_task.SmallSize());
         int i, j;
         for(i=0;i<current_task.Size();i++) for(j=0;j<current_task.Size();j++)
         {
            if ( current_task.Get(i, j) > 0) test_desk_2.SetItem(i, j, current_task.Get(i, j));
         }         
         
         variants_count = test_desk_2.GetTotalVariantsCount();
         
         if (test_desk_2.IsSolved())
         {
            hardness = 2;
         }
         else
         {
            hardness = 3;            
         }
      }
   }
}

void Sudoku::Generate(int difficulty_level){
   current_task = SudokuGenerator::Generate(current_task, difficulty_level);
}

int Sudoku::GetValueByChar( char c )
{
  if ( c > '0' && c <= '9' ) return c - '0';
  if ( c >= 'A' && c <= 'Z' ) return c - 'A' + 10;
  if ( c >= 'a' && c <= 'z' ) return c - 'a' + 10;
  return 0;
};

char Sudoku::GetCharByValue( int value )
{
  if ( value <= 0 ) return '_';
  if ( value < 10 ) return '0' + value;
  if ( value < 36 ) return 'A' + value - 10;
  return '_';
};
