/****************************************************************************

    QSudoku - a application for solving and generating sudoku puzzles
    Copyright (C) 2005  Yulia Shabunio

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

 ****************************************************************************/

#ifndef SUDOKU
#define SUDOKU 1

#include "SudokuSolver.h"
#include "SudokuGenerator.h"

class Sudoku
{
   SudokuTask current_task;
   int solution_count;
   SudokuDesk d1;
   SudokuDesk d2;   
public:
   Sudoku():current_task(3),solution_count(-1),d1(3),d2(3){};
   void LoadSudoku(const std::string filename);
   void SaveSudoku(const std::string filename, bool with_solution);   
   void Solve();
   void Generate(int difficulty_level);
   void GetInformation( int & solution_count, int & hardness, double & variants_count, int & count_of_steps, int & count_of_choices, std::list<SudokuChoice> & excess_items, std::list<SudokuSolveStep> & solution_history );   
   SudokuDesk & GetSolution(int number);
   
   const SudokuTask & GetCurrentTask() const { return current_task; };
   void SetCurrentTask(const SudokuTask & new_task ){ current_task = new_task; solution_count = -1; };   
   
   static int GetValueByChar( char c );
   static char GetCharByValue( int value );   
};

#endif
