#include <stdio.h>
#include <string.h>
#include "SudokuDesk.h"

unsigned char calc_bits( const unsigned long mask)
{
   unsigned char count = 0; unsigned long m = mask;
   for(; m != 0; count+= m & 1, m = m >> 1 );
   return count;
};

void print_bits(const unsigned long mask, const int bit_count)
{
   unsigned long m = mask;
   unsigned long print_mask = 1 << ( bit_count - 1 );
   for(int i=0;i<bit_count; i++)
   {
      if ( ( m & print_mask ) == 0 ){ printf("0"); }else{ printf("1"); }
      m = m << 1;
   }  
};
 
int get_value_by_mask(const unsigned long mask)
{
   int counter = 0; unsigned long value_mask = mask;
   for( ; value_mask > 1; value_mask >>=1, counter++);
   return counter + 1;
}

int get_one_of_value_by_mask(const unsigned long mask, int bit_count)
{
   if (bit_count == 0) return 0;
   if (bit_count == 1) return get_value_by_mask( mask );
   
   int random_bit = rand() % bit_count;

   int counter = 0; unsigned long value_mask = mask;
   for( ; value_mask > 0; value_mask >>=1, counter++)
   {
      if (value_mask & 1 > 0)
      {
         if (random_bit == 0) return counter + 1;
         random_bit --;
      }
   };
   // We never be here, but we mast return anything for compiler
   return 0;
}


SudokuDesk::SudokuDesk(const int small_size):observer(NULL),can_not_be_solved(false), use_normalization(true)
{
   if ( small_size > 5 )
   {
      throw SudokuException(QT_TR_NOOP("Sudoku lager than 5x5 is not supported"));
   }
   if ( small_size < 2 )
   {
      throw SudokuException(QT_TR_NOOP("Sudoku smaller than 2x2 is not supported"));
   }
   this->small_size = small_size;
   size = small_size * small_size;
   items = new unsigned long[size*size];
   var_count = new unsigned char[size*size];
   column_var_count = new unsigned char[size*size];
   row_var_count = new unsigned char[size*size];
   square_var_count = new unsigned char[size*size];      
   for(int i=0;i<size*size;i++) 
   { 
      items[i] = ( 1 << size ) - 1;
      var_count[i] = size;
      column_var_count[i] = size;
      row_var_count[i] = size;
      square_var_count[i] = size;
   }
};


SudokuDesk::SudokuDesk(const SudokuDesk & copy)
{
   small_size = copy.small_size;
   size = copy.size;
   items = new unsigned long[size*size];
   var_count = new unsigned char[size*size];
   row_var_count = new unsigned char[size*size];
   column_var_count = new unsigned char[size*size];
   square_var_count = new unsigned char[size*size];
   memcpy(items, copy.items, sizeof(unsigned long)*size*size);
   memcpy(var_count, copy.var_count, sizeof(unsigned char)*size*size);
   memcpy(row_var_count, copy.row_var_count, sizeof(unsigned char)*size*size);
   memcpy(column_var_count, copy.column_var_count, sizeof(unsigned char)*size*size);
   memcpy(square_var_count, copy.square_var_count, sizeof(unsigned char)*size*size);
   observer = copy.observer;
   can_not_be_solved = copy.can_not_be_solved;
   use_normalization = copy.use_normalization;
};


SudokuDesk & SudokuDesk::operator =(const SudokuDesk & copy)
{
   delete[] items;
   delete[] var_count;
   delete[] row_var_count;
   delete[] column_var_count;
   delete[] square_var_count;
   small_size = copy.small_size;
   size = copy.size;
   items = new unsigned long[size*size];
   var_count = new unsigned char[size*size];
   row_var_count = new unsigned char[size*size];
   column_var_count = new unsigned char[size*size];
   square_var_count = new unsigned char[size*size];
   memcpy(items, copy.items, sizeof(unsigned long)*size*size);
   memcpy(var_count, copy.var_count, sizeof(unsigned char)*size*size);
   memcpy(row_var_count, copy.row_var_count, sizeof(unsigned char)*size*size);
   memcpy(column_var_count, copy.column_var_count, sizeof(unsigned char)*size*size);
   memcpy(square_var_count, copy.square_var_count, sizeof(unsigned char)*size*size);
   observer = copy.observer; 
   can_not_be_solved = copy.can_not_be_solved; 
   return *this;
};


SudokuDesk::~SudokuDesk()
{
   delete[] items;
   delete[] var_count;
   delete[] row_var_count;
   delete[] column_var_count;
   delete[] square_var_count;
};

void SudokuDesk::EnableNormalization()
{
   use_normalization = true;
};

void SudokuDesk::DisableNormalization()
{
   use_normalization = false;
};

void SudokuDesk::apply_mask( const int i, const int j, const unsigned int value_mask)
{
   if (can_not_be_solved){ return; };
   int x = i*size+j;
   unsigned long prev_item = items[x];
   unsigned long new_item = prev_item & value_mask;
   if (new_item == 0){ 
      items[x] = 0; 
      can_not_be_solved = true; 
      var_count[x] = 0; 
      if (observer) observer->NextElement(i, j, 32, SudokuSolveObserver::impossible);      
      return; 
   };
   if (prev_item == new_item) return;
   
   items[x] = new_item;
   unsigned char bit_count = calc_bits(new_item);
   var_count[x] = bit_count;

   if ( bit_count == 1 )
   {
      if ( observer ) observer->NextElement(i, j, get_value_by_mask(new_item), SudokuSolveObserver::exact);
   }       
   
   unsigned long changed_bits = prev_item & ( ~ new_item );
   unsigned long saved_changed_bits = changed_bits;

   int small_square_number = (j-j%small_size+i/small_size)*size;
   int row_number = i*size;
   int column_number = j*size;
   int n;
   for(n=0;n<size; n++)
   {
      if (changed_bits & 1 != 0)
      {
         //Bit number c was cleared

         if ( (--row_var_count[row_number+n]) == 0)
         { 
            can_not_be_solved = true; 
            if (observer) observer->NextElement(i, j, n+1, SudokuSolveObserver::impossible_row);
            return; 
         };
         if ( (--column_var_count[column_number+n]) == 0)
         { 
            if (observer) observer->NextElement(i, j, n+1, SudokuSolveObserver::impossible_column);            
            can_not_be_solved = true; 
            return; 
         };
         if ( (--square_var_count[small_square_number+n]) == 0)
         { 
            if (observer) observer->NextElement(i, j, n+1, SudokuSolveObserver::impossible_square);            
            can_not_be_solved = true; 
            return; 
         };
      }
      changed_bits = changed_bits >> 1;
   }
   
   if ( bit_count == 1 )
   {
      extend_value( i, j, new_item);
      if (can_not_be_solved) return;      
   }       

   if (use_normalization)
   {   
      int r, c, q, k, l;
      for(n=0;n<size; n++)
      {
         if (saved_changed_bits & 1 != 0)
         {
            r = row_var_count[row_number+n];
            c = column_var_count[column_number+n];
            q = square_var_count[small_square_number+n];
   
            unsigned long mask = 1 << n;
            if (r == 1)
            {
               for (k=0; k<size;k++)
               {
                  if ((items[i*size+k] & mask) > 0)
                  {
                     apply_mask(i, k, mask);
                     if (can_not_be_solved) return;
                  }
               }
            }
            if (c == 1)
            {
               for (k=0; k<size;k++)
               {
                  if ((items[k*size+j] & mask) > 0)
                  {
                     apply_mask(k, j, mask);
                     if (can_not_be_solved) return;
                  }
               }
            }
            if (q == 1)
            {
               int si = (i/small_size)*small_size, sj = (j/small_size)*small_size;
               for (k=si; k<si+small_size;k++)for (l=sj; l<sj+small_size;l++)
               {
                  if ((items[k*size+l] & mask) > 0)
                  {
                     apply_mask(k, l, mask);
                     if (can_not_be_solved) return;
                  }
               }
            }
   
         }
         saved_changed_bits = saved_changed_bits >> 1;
      }
   }
}

/*
  This function extend influence of exact value to row, column and small square.
  If new exact value (with only one bit) appears, function extend_value called for one.
*/
void SudokuDesk::extend_value( const int i, const int j, unsigned long value_mask)
{
   if (observer) observer->NextElement(i, j, get_value_by_mask(value_mask), SudokuSolveObserver::exact);

   value_mask = ~value_mask;
   int k, l;
   
   for(k=0;k<size;k++) 
   {
      // Fill Row
      if (k != i) 
      { 
         apply_mask( k, j, value_mask);
         if (can_not_be_solved) return;
      } 
      // Fill column         
      if (k != j)
      {
         apply_mask( i, k, value_mask);
         if (can_not_be_solved) return;
      } 
   }      
      
   //Small square
   int si = ( i / small_size ) * small_size, sj = ( j / small_size ) * small_size;

   for( k = si; k<si+small_size; k++)   
      for( l = sj; l<sj+small_size; l++)  
         if ( k != i && l != j ) //this is not error. LIne and column in the square we filled early
         { 
            apply_mask( k, l, value_mask);
            if (can_not_be_solved) return;
         }
};
   
void SudokuDesk::SetItem(const int i, const int j, const int value)
{
  bool dump;
  SetItem( i, j, value, dump);
}

void SudokuDesk::SetItem(const int i, const int j, const int value, bool & NewInformation)
{
   if ( ( i < 0 ) || ( i >= size ) ) throw SudokuException(QT_TR_NOOP("i out of range"));
   if ( ( j < 0 ) || ( j >= size ) ) throw SudokuException(QT_TR_NOOP("j out of range"));
   
   unsigned long value_mask = 1 << ( value - 1 );

   if ( items[i*size+j] == value_mask )
   {
      NewInformation = false;
      return;
   }
   NewInformation = true;

   apply_mask( i, j, value_mask);
};

void SudokuDesk::ExcludeItem(const int i, const int j, const int Value)
{
   if ( ( i < 0 ) || ( i >= size ) ) throw SudokuException(QT_TR_NOOP("i out of range"));
   if ( ( j < 0 ) || ( j >= size ) ) throw SudokuException(QT_TR_NOOP("j out of range"));

   unsigned long value_mask = 1 << ( Value - 1 );

   apply_mask( i, j, ~value_mask);
};

int SudokuDesk::GetItem(const int i, const int j) const
{
   if ( ( i < 0 ) || ( i >= size ) ) throw SudokuException(QT_TR_NOOP("i out of range"));
   if ( ( j < 0 ) || ( j >= size ) ) throw SudokuException(QT_TR_NOOP("j out of range"));

   unsigned long value_mask = items[i*size+j];
   if ( calc_bits(value_mask) == 1 )
   {
      return get_value_by_mask(value_mask);
   } else {
      return 0;
   }
};

int SudokuDesk::GetOneOfItems(const int i, const int j) const
{
   return get_one_of_value_by_mask(items[i*size+j], var_count[i*size+j]);
};

int SudokuDesk::GetFirstOneOfItems(const int i, const int j) const
{
   return get_value_by_mask(items[i*size+j]);
};

int SudokuDesk::GetBestOneOfItems(const int i, const int j) const
{
   int bit_count = var_count[i*size+j];
   unsigned long mask =  items[i*size+j];
   
   if (bit_count == 0) return 0;
   if (bit_count == 1) return get_value_by_mask( mask );
   
   int counter = 0; 
   unsigned long value_mask = mask;
   int small_square_number = (j-j%small_size+i/small_size)*size;
   int row_number = i*size;
   int column_number = j*size;   
   int minimum_variants = size + 1;
   int value = 0;
   
   for( ; value_mask > 0; value_mask >>=1, counter++)
   {
      if (value_mask & 1 > 0)
      {
         if ( minimum_variants > row_var_count[row_number+counter] )
         {
            minimum_variants = row_var_count[row_number+counter];
            value = counter + 1;   
         }
         if ( minimum_variants > column_var_count[column_number+counter] )
         {
            minimum_variants = column_var_count[column_number+counter];
            value = counter + 1;   
         }
         if ( minimum_variants > square_var_count[small_square_number+counter] )
         {
            minimum_variants = square_var_count[small_square_number+counter];
            value = counter + 1;   
         }
      }
   };
   return value;
}


bool SudokuDesk::GetMinVaryItem(int & i, int & j) const
{
   int min_vary;
   min_vary = size + 1; i=-1; j=-1; //If this values will not change, then sudoku solved alredy.
  
   for(int k=0; k<size; k++) for(int l=0; l<size; l++)
   {
      if (var_count[k*size+l] < min_vary && var_count[k*size+l] > 1)
      {
         min_vary = var_count[k*size+l]; i=k; j=l;
         if (min_vary == 2) return true;
      }
   }
   if (i==-1 && j==-1) return false;
   return true;
};


bool SudokuDesk::GetMaxVaryItem(int & i, int & j) const
{
   int min_vary;
   min_vary = 0; i=-1; j=-1; //If this values will not change, then sudoku solved alredy.
  
   for(int k=0; k<size; k++) for(int l=0; l<size; l++)
   {
      if (var_count[k*size+l] > min_vary && var_count[k*size+l] > 1)
      {
         min_vary = var_count[k*size+l]; i=k; j=l;
         if (min_vary == small_size) return true;
      }
   }
   if (i==-1 && j==-1) return false;
   return true;
};

bool SudokuDesk::GetFirstVaryItem(int & i, int & j) const
{
   for(int k=0; k<size; k++) for(int l=0; l<size; l++)
   {
      if (var_count[k*size+l] > 1)
      {
          i=k;
          j=l;
          return true;
      }
   }
   return false;
};

bool SudokuDesk::GetRandomUnknownItem(int & i, int & j) const
{
   int c, vary_items = 0, item;
   
   for(c = 0; c<size*size; c++)
   {
      if (var_count[c] > 1) vary_items++;
   }
   if (!vary_items) return false;
   item = rand() % vary_items;  // j get a random value between 0 and maxrand

   for(c = 0; c<=size*size; c++)
   {
      if (var_count[c] > 1)
      {
         if (item == 0)
         {
            i = c / size;
            j = c % size;
            return true;
         }
         item--;
      }
   }
   return false;
};

double SudokuDesk::GetTotalVariantsCount() const
{

   int i; double cnt = 1;
   for(i=0; i<size*size; i++)
   {
      if (var_count[i] == 0) return 0;
      cnt = cnt * (var_count[i]);
   }
   return cnt;
};

bool SudokuDesk::Unsoluble() const
{
   return can_not_be_solved;
};

bool SudokuDesk::IsSolved() const
{
   if (can_not_be_solved) return false;
   for(int i=0;i<size*size;i++) if (var_count[i] != 1) return false;
   return true;
};

void SudokuDesk::SetObserver(SudokuSolveObserver * o){
   observer = o;
};

void SudokuDesk::Print() const
{
   int i; int j;
  
  for(i=0;i<size;i++)
  { 
    printf("%d:", i);
    for(j=0;j<size;j++)
    {
       print_bits(items[i*size+j], size);
       printf("(%d) ", var_count[i*size+j] );
    }
    printf("\n");
  }
  printf("\n");
  
  printf("c:n\tr:n\tq:n\n");
  for(i=0;i<size;i++)
  { 
     printf("%d(", i);
     for(j=0;j<size;j++)
     {
        printf("%d,", column_var_count[i*size+j] );
     }
     printf(")\t%d(", i);
     for(j=0;j<size;j++)
     {
        printf("%d,", row_var_count[i*size+j] );
     }           
     printf(")\t%d(", i);
     for(j=0;j<size;j++)
     {
        printf("%d,", square_var_count[i*size+j] );
     }                      
     printf(")\n");
  }  
  printf("\n\n");    
};
