/****************************************************************************

    QSudoku - a application for solving and generating sudoku puzzles
    Copyright (C) 2005  Yulia Shabunio

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

 ****************************************************************************/
#ifndef SUDOKU_DESK
#define SUDOKU_DESK 1

#include "Tools.h"


class SudokuSolveObserver
{
public: 
   enum SudokuStepType
   {
      exact
    , questionable
    , impossible
    , impossible_column
    , impossible_row
    , impossible_square
    , aborted
    , unknown    
   };   
   
    virtual void NextElement(int i, int j, int value, SudokuStepType type) = 0;
};

class SudokuDesk
{
   
private:
   int small_size;
   int size;
   unsigned long * items;
   unsigned char * var_count;
  
   unsigned char * column_var_count;
   unsigned char * row_var_count;
   unsigned char * square_var_count;
   
   SudokuSolveObserver * observer;
   
   bool can_not_be_solved;
   
   bool use_normalization;

   void extend_value( const int i, const int j, const unsigned long value_mask);
   void apply_mask( const int i, const int j, const unsigned int value_mask);
public:
   
   //small_size is the size of small square. May by 2, 3, 4, 5.
   SudokuDesk(const int small_size);
   ~SudokuDesk();

   //Creation copy of desk   
   SudokuDesk(const SudokuDesk & copy);
   SudokuDesk & operator =(const SudokuDesk & copy);
  
   //Set object for observation
   void SetObserver(SudokuSolveObserver * o);

  // i, j is zero-based parameters.
  // value is one-based.
  // 0 <= i < size, 0<= j < size, 1 <= value <= size
  // size = small_size*small_size

   //get size of desk
   int GetSize() const {return size;};
   //get item of (I, J) if it has exact number or 0
   int GetItem(const int i, const int j) const;
   //get one of possible values at (i, j) or 0 if it is not exists;
   int GetOneOfItems(const int i, const int j) const;
   //get one of possible values at (i, j) or 0 if it is not exists;
   int GetFirstOneOfItems(const int i, const int j) const;
   //get best of possible values at (i, j) or 0 if it is not exists;
   int GetBestOneOfItems(const int i, const int j) const;
   
   //Set item value and know, is it new information for desk.
   void SetItem(const int i, const int j, const int Value, bool & NewInformation);
   //Set item value
   void SetItem(const int i, const int j, const int Value);
   //Exclude Value from possible values of I, J item.
   void ExcludeItem(const int i, const int j, const int Value);

   //Get I and J of first questionable item. Return false, if it is impossible
   bool GetFirstVaryItem(int & i, int & j) const;
   //Get I and J of questionable item with minimum variants count. Return false, if it is impossible
   bool GetMinVaryItem(int & i, int & j) const;
   //Get I and J of questionable item with maximum variants count. Return false, if it is impossible
   bool GetMaxVaryItem(int & i, int & j) const;
   
   //Get I and J of random questionable item. Return false, if it is impossible
   bool GetRandomUnknownItem(int & i, int & j) const;

   //Is the desk unsoluble?
   bool Unsoluble() const;
   //Is then desk solved
   bool IsSolved() const;
   //get total count of choice variants;
   double GetTotalVariantsCount() const;

   //Enable all possibilities of SudokuDesk for faster solve.
   void EnableNormalization(); 
   //Disable normalization for difficulty estimation or new puzzle generation
   void DisableNormalization(); 

   //Print desk bits. For debug only.
   void Print() const;
};

#endif
