#include "SudokuGenerator.h"
#include <algorithm>
#include <vector>

SudokuTask SudokuGenerator::Generate(const SudokuTask & source, int desired_difficuly )
{
   if (desired_difficuly < 0 || desired_difficuly > 2 ) throw SudokuException(QT_TR_NOOP("Unknown value of desired_difficuly"));
   
   SudokuTask current_task = source;

   int limit = 1000;   
   SudokuSolver * sl = new SudokuSolver(current_task);
   sl->SetRandomSolveMode(true);
   sl->SetStepLimit(limit);
   int sollve_result = sl->Solve();
   
   while ( sollve_result == -1 )
   {
      limit+=10;
      delete sl;
      sl = new SudokuSolver(current_task);
      sl->SetRandomSolveMode(true);
      sl->SetStepLimit(limit);
      sollve_result = sl->Solve();
   }
   
   if (!sollve_result)
   {
      throw SudokuException(QT_TR_NOOP("It is impossible generate SUDOKU on this base"));
   }
   SudokuDesk solution = sl->GetFirstSolve();
   delete sl;
   
   int i, j, value;
   std::vector<SudokuChoice> task_as_vector; 
   task_as_vector.reserve(source.Size()*source.Size());

   for (i=0;i<source.Size();i++) for(j=0;j<source.Size();j++)
   {
      value = solution.GetItem(i, j);
      current_task.Set(i, j, value);
      task_as_vector.push_back(SudokuChoice(i, j, value));
   }
   std::vector<SudokuChoice>::const_iterator it;   
   random_shuffle( task_as_vector.begin(), task_as_vector.end() );
   
   SudokuDesk test_desk(current_task.SmallSize()); bool new_information;
   
   if (desired_difficuly == 0 ) test_desk.DisableNormalization();

   for(it=task_as_vector.begin();it !=task_as_vector.end();it++)
   {
      test_desk.SetItem(it->i, it->j, it->value, new_information);      
      if (!new_information && source.Get(it->i, it->j) == 0 ) current_task.Set(it->i, it->j, 0);
   }
   
   SudokuDesk next_test_desk(current_task.SmallSize()); 
   if (desired_difficuly == 0 ) next_test_desk.DisableNormalization();   

   for(i=0;i<current_task.Size();i++)   
   for(j=0;j<current_task.Size();j++)
   {
      if ( current_task.Get(i, j)>0)
      {
         next_test_desk.SetItem(i, j, current_task.Get(i, j), new_information);      
         if (!new_information && source.Get(i, j) == 0 ) current_task.Set(i, j, 0);
      }
   }
   
   if ( desired_difficuly < 2 ) return current_task;
  
   int dec_counter = 0;
   for(it=task_as_vector.begin();it !=task_as_vector.end();it++)
   {
      if (!current_task.Get(it->i, it->j)) continue;
      if (source.Get(it->i, it->j) != 0) continue;
      
      current_task.Set(it->i, it->j, 0);
      
      SudokuSolver * sl = new SudokuSolver(current_task);
      sl->SetStepLimit(500);
      int solution_count = sl->Solve();
      if (!solution_count)
      {
         throw SudokuException(QT_TR_NOOP("It is impossible generate SUDOKU on this base"));
      }      
      if (solution_count > 1 || solution_count == -1 )
      {
         current_task.Set(it->i, it->j, it->value);
      }else
      {
         dec_counter++;
         if (dec_counter >= current_task.Size()) break;
      }
      delete sl;      
   }

   return current_task;      
}
